/* File "datagram.h", Light Sockets - Copyright (C) Matt Slot, 1996           */
/* Foundation layer for the Light Sockets networking abstraction routines.    */

#ifndef __DATAGRAM_HEADER__
#define __DATAGRAM_HEADER__

#ifndef __STD_TYPES_HEADER__
#include "stdtypes.h"
#endif /* __STD_TYPES_HEADER__ */

#ifndef __SOCKET_HEADER__
#include "socket.h"
#endif /* __SOCKET_HEADER__ */

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Preprocessor Defines */


/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Enum/Structure/Class Definitions */

typedef class DatagramSocket : public Socket {

friend class NetworkStack;
friend class MacTCPStack;

public:
							DatagramSocket	(void);
	virtual					~DatagramSocket	(void);


	/* Primary member functions */
	virtual SocketResult	Create			(SocketType socketType);
	virtual SocketResult	Tickle			(void);
	virtual SocketResult	Dispose			(void);
	
	/* Addition member functions */
	SocketResult 			Write			(Byte8 *dataPtr, UInt32 dataLen,
												SocketAddressPtr address);
	
	SocketResult			GetWriteProc	(DatagramWriteProc *proc,
												void **refcon);
	SocketResult			SetWriteProc	(DatagramWriteProc proc,
												void *refcon);
	SocketResult			GetReadProc		(DatagramReadProc *proc,
												void **refcon);
	SocketResult			SetReadProc		(DatagramReadProc proc,
												void *refcon);

//protected:

	SocketResult 			CallWriteProc	(Byte8 *dataPtr, UInt32 dataLen,
												SocketAddressPtr address);
	SocketResult 			CallReadProc	(Byte8 *dataPtr, UInt32 dataLen,
												SocketAddressPtr address);

private:

	static SocketResult 	WriteStub		(class DatagramSocket *socket,
												void *refcon,
												Byte8 *dataPtr, UInt32 dataLen,
												SocketAddressPtr address);
	static SocketResult 	ReadStub		(class DatagramSocket *socket,
												void *refcon,
												Byte8 *dataPtr, UInt32 dataLen,
												SocketAddressPtr address);

	/* Callbacks */
	DatagramWriteProc		writeProc;		/* Write routine chain and data */
	void *					writeRefcon;	/* Write routine chain and data */
	DatagramReadProc		readProc;		/* Read routine chain and data */
	void *					readRefcon;		/* Read routine chain and data */
	} DatagramSocket, *DatagramSocketRef;

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Function Prototypes */


/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

#endif /* __DATAGRAM_HEADER__ */

