/* File "client.cpp", ISR/Reggie - Copyright (C) Matt Slot+Mike Monan, 1996   */
/* Core source file for the reggie tracker implementation.                    */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <sys/time.h>
#include <time.h>

#include "stdtypes.h"
#include "stddebug.h"

#include "client.h"
#include "datagram.h"
#include "platform.h"
#include "reggie.h"
#include "socket.h"

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Global Declarations */

UInt32				gTransID = 0;
UInt32				gTimeout = 0;
UInt32				gRegCount = 0;
RegEntry			gRegStore[kRegEntryCount];
DatagramReadProc	gSaveReadProc = 0;
void *				gSaveReadRefcon = 0;

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

#ifdef NEVER

void main() {
	SInt32 error = 0;
	
#if defined(PLATFORM_MAC)
	printf("Initializing Quickdraw\n");
#endif

	qThrowIfError(QueryTracker(0x12345678, 0, 100, NULL, NULL, NULL, 
			"sils.umich.edu"), 0);
	qThrowIfError(DumpRegEntries(), 0);

/* CATCH */
	qCatch();

#if defined(PLATFORM_MAC)
	printf("\n--- End of run ---\n");
#endif
	}

#endif NEVER
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 DumpRegEntries() {
	SInt32 error = eSocketNoError;
	UInt32 i;
	
	for(i=0; i<gRegCount; i++)
		printf("%sEntry %ld: %s\n%s\n%s\n", ((i) ? "-----\n" : ""), 
				i, gRegStore[i].serviceType, gRegStore[i].serviceName,
				gRegStore[i].serviceInfo);

/* CATCH */
	qCatch();

	return(error);
	}
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 QueryTracker(UInt32 transactionID, UInt32 startIndex, UInt32 endIndex,
		Char8 *typeMask, Char8 *nameMask, Char8 *infoMask, Char8 *trackerName) {
	SInt32 error = eSocketNoError;
	Bool8 bound=false;
	UInt32 i, offset;
	Packet sndPacket;
	DatagramSocket socket;
	SocketAddress serverAddr;
	
	ResolveNetworkAddress(eSocketTypeUDP, trackerName, &serverAddr);
	serverAddr.buffer.tcpip.port = kTrackerPort;


	qThrowIfError(socket.Create(serverAddr.type), "Can't create client port");
	qThrowIfError(socket.Bind(0, 0), "Can't bind client");
	bound = true;

	qThrowIfError(socket.GetReadProc((DatagramReadProc *) &gSaveReadProc,
			&gSaveReadRefcon), 0);
	qThrowIfError(socket.SetReadProc((DatagramReadProc) HandlePacket, 0), 0);

#ifdef _DEBUG
//	socket.EnableLogging("client.log", true);
#endif

	sndPacket.query.transactionType = eTransactionQuery;
	sndPacket.query.transactionID = transactionID;
	sndPacket.query.reserved[0] = 0;
	sndPacket.query.reserved[1] = 0;
	sndPacket.query.reserved[2] = 0;
	sndPacket.query.reserved[3] = 0;
	sndPacket.query.indexStart = startIndex;
	sndPacket.query.indexEnd = (!endIndex || (startIndex <= endIndex)) ?
			endIndex : startIndex;

	offset = 0;
	strcpy(sndPacket.query.serviceType + offset, (typeMask) ? typeMask : "");
	offset += ((typeMask) ? strlen(typeMask) : 0) + 1;
	strcpy(sndPacket.query.serviceType + offset, (nameMask) ? nameMask : "");
	offset += ((nameMask) ? strlen(nameMask) : 0) + 1;
	strcpy(sndPacket.query.serviceType + offset, (infoMask) ? infoMask : "");
	offset += ((infoMask) ? strlen(infoMask) : 0) + 1;

	gRegCount = 0;
    qThrowIfError(socket.Write((Byte8 *) &sndPacket, 
			sndPacket.query.serviceType - (Byte8 *) &sndPacket + offset, 
			&serverAddr), 0);
	
	gTransID = transactionID;
#if defined(PLATFORM_MAC)
	for(gTimeout=2000; gTimeout>0; gTimeout--)
#else
	for(gTimeout=10000; gTimeout>0; gTimeout--)
#endif 
		qThrowIfError(socket.Tickle(), 0);

/* CATCH */
	qCatch();

	if (bound) qTraceIfError(socket.Unbind(), 0);
	qTraceIfError(socket.Dispose(), 0);
	return(error);
	}
	
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 HandlePacket(DatagramSocket *socket, void *refcon,
		PacketPtr rcvPacket, UInt32 packetLen, SocketAddressPtr remoteAddr) {
	SInt32 error = eSocketNoError;
	UInt32 i;
	Char8 *type, *name, *info;
	
	if (rcvPacket->answer.transactionID != gTransID) 
		qThrowMsg("Unexpected transaction ID");
	
	type = rcvPacket->answer.serviceType;
	name = type + strlen(type) + 1;
	info = name + strlen(name) + 1;

	i = gRegCount++;
	memcpy(&gRegStore[i].serviceAddr, &rcvPacket->answer.serviceAddr,
			sizeof(gRegStore[i].serviceAddr));
	strcpy(gRegStore[i].serviceType, type);
	strcpy(gRegStore[i].serviceName, name);
	strcpy(gRegStore[i].serviceInfo, info);

	/* Reset our counter */
	if (gTimeout < 500) gTimeout += 100;

/* CATCH */
	qCatch();
	return(error);
	}
