/* File "register.cpp", ISR/Reggie - Copyright (C) Matt Slot+Mike Monan, 1996 */
/* ISR/Reggie regististration utility for networking apps                     */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "stdtypes.h"
#include "stddebug.h"

#include "platform.h"
#include "reggierc.h"
#include "socket.h"

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Function Prototypes */


/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 LoadRCFile(Char8 *filename, Char8 **filedata) {
	SInt32 error = eSocketNoError;
	SInt32 filelen;
	Char8 *filetemp = 0;
	FILE *rcfile = 0;
	
	qThrowIfNull(filename, eSocketErrBadParam, kSocketErrBadParamStr);
	qThrowIfNull(filedata, eSocketErrBadParam, kSocketErrBadParamStr);
	qThrowIfNull(rcfile = fopen(filename, "r"), errno, 
			"Error opening rc config file");
	
	*filedata = 0;
	
	qThrowIfNull(filetemp = new Char8[MAX_FILE_SIZE], eSocketErrOutOfMem, 
			kSocketErrOutOfMemStr);
	
	filelen = fread(filetemp, 1, MAX_FILE_SIZE, rcfile);
	qThrowIfTrue(filelen < 0, filelen, "File too long, dorx");

	qThrowIfNull(*filedata = new Char8[filelen+1], eSocketErrOutOfMem, 
			kSocketErrOutOfMemStr);
	memcpy(*filedata, filetemp, filelen);
	(*filedata)[filelen] = 0;
	
/* CATCH */
	qCatch();
	
	if (rcfile) qTraceIfError(fclose(rcfile), "Error closing rc config file");
	if (filetemp) delete filetemp;
	
	return(error);
	}
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 UnloadRCFile(Char8 *filedata) {
	SInt32 error = eSocketNoError;
	SInt32 filelen;
	Char8 *filetemp = 0;
	FILE *rcfile = 0;
	
	qThrowIfNull(filedata, eSocketErrBadParam, kSocketErrBadParamStr);
	delete filedata;
	
/* CATCH */
	qCatch();
	return(error);
	}
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

UInt32 GetNextService(Char8 *filedata, UInt32 offset, SocketAddressPtr address,
		Char8 *serviceType, Char8 *serviceName, Char8 *serviceInfo) {
	SInt32 error = eSocketNoError;
	Char8 line[ARBITRARY_BUFFER_SIZE], *token;
	UInt32 i, j, offSaved, typeSaved;
	Bool8 found;
	
	qThrowIfNull(filedata, eSocketErrBadParam, kSocketErrBadParamStr);
	qThrowIfNull(address, eSocketErrBadParam, kSocketErrBadParamStr);
	qThrowIfNull(serviceType, eSocketErrBadParam, kSocketErrBadParamStr);
	qThrowIfNull(serviceName, eSocketErrBadParam, kSocketErrBadParamStr);
	qThrowIfNull(serviceInfo, eSocketErrBadParam, kSocketErrBadParamStr);

	/* Read line 1 */
	for(i=0; i<3; i++) {
		for(found=false; !found; offset++) {
			for(offSaved=offset; filedata[offset] && (filedata[offset] != '\r') 
					&& (filedata[offset] != '\n'); offset++);
			if (filedata[offset] == 0) return(0);
				
			qThrowIfTrue(offset-offSaved >= ARBITRARY_BUFFER_SIZE, -1,
					"Line too long");
			memcpy(line, filedata+offSaved, offset-offSaved);
			line[offset-offSaved] = 0;
			if ((filedata[offset] == '\r') && (filedata[offset+1] == '\n')) 
				offset++;
			
			if (strlen(line) && (line[0] != '#')) found = true;
			}

		if (i==0) {
			for(j=0, token = strtok(line, ":\t"); token; 
					j++, token = strtok(0, ":\t")) {
				
				if (j==0) {
					if (!strcmp("UDP", token) || !strcmp("udp", token))
						typeSaved = eSocketTypeUDP;
					  else if (!strcmp("TCP", token) || !strcmp("tcp", token))
						typeSaved = eSocketTypeTCP;
					  else if (!strcmp("IP", token) || !strcmp("ip", token))
						typeSaved = eSocketTypeRawIP;
					  else qThrowErr(-1, "Invalid Protocol");
					}
				  else if (j==1) {
					qThrowIfError(ResolveNetworkAddress(eSocketTypeUDP,
							token, address), 0);
					address->type = typeSaved;
				  	}
				  else if (j==2) address->buffer.tcpip.port = atol(token);
				  else if (j==3) strcpy(serviceType, token);
				}
			}
		  else if (i==1) strcpy(serviceName, line);
		  else if (i==2) strcpy(serviceInfo, line);
		}

/* CATCH */
	qCatch();
	return((error) ? 0 : offset+1);
	}
	
