/* File "tracker.cpp", ISR/Reggie - Copyright (C) Matt Slot+Mike Monan, 1996  */
/* Core source file for the reggie tracker implementation.                    */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>

#include "stdtypes.h"
#include "stddebug.h"

#include "datagram.h"
#include "reggierc.h"
#include "reggie.h"
#include "socket.h"
#include "store.h"
#include "tracker.h"

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Global Declarations */

c_store 			gDBStore(sizeof(DBData));
DatagramReadProc	gSaveReadProc = 0;
void *				gSaveReadRefcon = 0;

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

void main() {
	SInt32 error = 0;
	
#if defined(PLATFORM_MAC)
	printf("Initializing Quickdraw\n");
#endif

	InitDB();
	IdleLoop();

#if defined(PLATFORM_MAC)
	printf("\n--- End of run ---\n");
#endif
	}
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 InitDB() {
	SInt32 error = eSocketNoError;
	UInt32 fileoffset = 0;
	Char8 *filedata = 0;
	Char8 serviceType[20];
	Char8 serviceName[ARBITRARY_BUFFER_SIZE];
	Char8 serviceInfo[ARBITRARY_BUFFER_SIZE];
	SocketAddress remoteAddr;
	
	qThrowIfError(LoadRCFile("reggierc", &filedata), 0);
	while(fileoffset = GetNextService(filedata, fileoffset, &remoteAddr,
			serviceType, serviceName, serviceInfo)) {
		UInt32 strOffset;
		ServiceAddress serviceAddr;
		
		serviceAddr.hostProtFamily = 
				(remoteAddr.type == eSocketTypeUDP) ? PF_UDP : PF_TCP;
		serviceAddr.hostAddrFamily = AF_INET;
		serviceAddr.hostPort = remoteAddr.buffer.tcpip.port;
		serviceAddr.hostAddress = remoteAddr.buffer.tcpip.host;
		memset(serviceAddr.data, 0, sizeof(serviceAddr.data));
	
		qThrowIfError(RegisterService(eServicePermanent, &serviceAddr, 
				serviceType, serviceName, serviceInfo), 0);
		}
	qThrowIfError(UnloadRCFile(filedata), 0);

/* CATCH */
	qCatch();
	return(error);
	}
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 IdleLoop() {
	SInt32 error = eSocketNoError;
	Bool8 done = false;
	Bool8 bound = false;
	SocketAddress localAddr, boundAddr;
	DatagramSocket socket;
	struct timeval timeout;
	
	localAddr.type = eSocketTypeUDP;
	localAddr.length = sizeof(localAddr.buffer.tcpip);
	localAddr.buffer.tcpip.type = AF_INET;
	localAddr.buffer.tcpip.port = kTrackerPort;
	localAddr.buffer.tcpip.host = 0;
	memset(localAddr.buffer.tcpip.data, 0, sizeof(localAddr.buffer.tcpip.data));

	qThrowIfError(socket.Create(localAddr.type), "Can't create tracker port");
	qThrowIfError(socket.Bind(&localAddr, &boundAddr), "Can't bind tracker");
	bound = true;

	qThrowIfError(socket.GetReadProc((DatagramReadProc *) &gSaveReadProc,
			&gSaveReadRefcon), 0);
	qThrowIfError(socket.SetReadProc((DatagramReadProc) HandlePacket, 0), 0);

#ifdef _DEBUG
	socket.EnableLogging("tracker.log", true);
#endif

	while(!done) {
		timeout.tv_sec = 0; timeout.tv_usec = 250000;
		select(getdtablesize(),0,0,0, &timeout);
		qThrowIfError(socket.Tickle(), 0);
		TickleService();
		}

/* CATCH */
	qCatch();

	if (bound) qTraceIfError(socket.Unbind(), 0);
	qTraceIfError(socket.Dispose(), 0);
	return(error);
	}
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 HandlePacket(DatagramSocket *socket, void *refcon,
		PacketPtr rcvPacket, UInt32 packetLen, SocketAddressPtr remoteAddr) {
	SInt32 error = eSocketNoError;
	Char8 *type, *name, *info;
	ServiceAddress serviceAddr;
	
	if ((packetLen < 8) || !rcvPacket) {
		qThrowIfError(SendAckPacket(socket, remoteAddr, 0), 0);
		}
	  else switch(rcvPacket->any.transactionType) {
		case eTransactionAck:
			qThrowMsg("Unexpected transaction type");
			break;

		case eTransactionPing:
			qThrowIfError(SendAckPacket(socket, remoteAddr, 
					rcvPacket->any.transactionID), 0);
			break;
	
		case eTransactionRegister:
			type = rcvPacket->reg.serviceType;
			name = type + strlen(type) + 1;
			info = name + strlen(name) + 1;
			qThrowIfTrue(((Byte8 *) info+strlen(info)+1) - (Byte8 *) rcvPacket >
					packetLen, 0, "Bad Packet Structure");

			qThrowIfError(RegisterService(rcvPacket->reg.serviceTTL, 
					&rcvPacket->reg.serviceAddr, type, name, info), 0);
			qThrowIfError(SendAckPacket(socket, remoteAddr, 
					rcvPacket->any.transactionID), 0);
			break;

		case eTransactionUnregister:
			type = rcvPacket->reg.serviceType;
			name = type + strlen(type) + 1;
			info = name + strlen(name) + 1;
			qThrowIfTrue(((Byte8 *) info+strlen(info)+1) - (Byte8 *) rcvPacket >
					packetLen, 0, "Bad Packet Structure");

			qThrowIfError(DeleteService(&rcvPacket->reg.serviceAddr,
					type, name, info), 0);
			qThrowIfError(SendAckPacket(socket, remoteAddr, 
					rcvPacket->any.transactionID), 0);
			break;
	
		case eTransactionQuery:
			type = rcvPacket->query.serviceType;
			name = type + strlen(type) + 1;
			info = name + strlen(name) + 1;
			qThrowIfError(FindServices(socket, remoteAddr,
					rcvPacket->query.transactionID, rcvPacket->query.indexStart,
					rcvPacket->query.indexEnd, type, name, info), 0);
			break;
	
		case eTransactionAnswer:
			qThrowMsg("Unexpected transaction type");
			break;
	
		default:
			qThrowMsg("Unexpected transaction type");
			break;
		}
	
/* CATCH */
	qCatch();

	return(error);
	}
		
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 RegisterService(UInt32 serviceTTL, ServiceAddressPtr serviceAddr, 
		Char8 *serviceType, Char8 *serviceName, Char8 *serviceInfo) {
	SInt32 error = eSocketNoError;
	UInt32 i;
	DBData *walkData;
	DBData newData;
	
	for(i=0; walkData = (DBDataPtr) gDBStore.get_nth_elem(i); i++) {
		if (memcmp(serviceAddr, &walkData->serviceAddr, 
				sizeof(*serviceAddr) - sizeof(serviceAddr->data))) continue;
		if (strcmp(serviceType, walkData->serviceType)) continue;
		if (strcmp(serviceName, walkData->serviceName)) continue;
		if (strcmp(serviceInfo, walkData->serviceInfo)) continue;
		
		walkData->servicePing = time(0);
		return(error);
        }

	newData.servicePing = time(0);
	newData.serviceTTL = serviceTTL;
	memcpy(&newData.serviceAddr, serviceAddr, sizeof(newData.serviceAddr));
	strcpy(newData.serviceType, serviceType);
	strcpy(newData.serviceType, serviceType);
	strcpy(newData.serviceName, serviceName);
	strcpy(newData.serviceInfo, serviceInfo);
	qThrowIfError(gDBStore.append_elem((Byte8 *) &newData), 0);

/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 TickleService() {
	SInt32 error = eSocketNoError;
	UInt32 i;
	DBData *walkData;
	
    for(i=0; walkData = (DBDataPtr) gDBStore.get_nth_elem(i); i++) {
		if (walkData->serviceTTL == eServicePermanent) continue;
		if (walkData->servicePing + walkData->serviceTTL >= time(0)) continue;
		gDBStore.remove_elem(i--); /* Reset our walker index */
        }

/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 DeleteService(ServiceAddressPtr serviceAddr, 
		Char8 *serviceType, Char8 *serviceName, Char8 *serviceInfo) {
	SInt32 error = eSocketNoError;
	UInt32 i;
	DBData *walkData;
	
	for(i=0; walkData = (DBDataPtr) gDBStore.get_nth_elem(i); i++) {
		if (memcmp(serviceAddr, &walkData->serviceAddr, 
				sizeof(serviceAddr) - sizeof(serviceAddr->data))) continue;
		if (strcmp(serviceType, walkData->serviceType)) continue;
		if (strcmp(serviceName, walkData->serviceName)) continue;
		if (strcmp(serviceInfo, walkData->serviceInfo)) continue;
		
		gDBStore.remove_elem(i);
		return(error);
        }

/* CATCH */
	qCatch();
	return(0);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 FindServices(DatagramSocketRef socket, SocketAddressPtr remoteAddr,
		UInt32 transactionID, UInt32 startIndex, UInt32 stopIndex,
		Char8 *serviceType, Char8 *serviceName, Char8 *serviceInfo) {
	SInt32 error = eSocketNoError;
	Bool8 found;
	UInt32 i, offset;
	DBData *walkData;
	Packet sndPacket;

	for(i=startIndex; (i<stopIndex) && 
			(walkData = (DBDataPtr) gDBStore.get_nth_elem(i)); i++) {
		if (*serviceType && 
				strcmp(serviceType, walkData->serviceType)) continue;
		//if (strcmp(serviceName, walkData->serviceName)) continue;
		//if (strcmp(serviceInfo, walkData->serviceInfo)) continue;
		
	
		sndPacket.answer.transactionType = eTransactionAnswer;
		sndPacket.answer.transactionID = transactionID;
		sndPacket.answer.reserved[0] = 0;
		sndPacket.answer.serviceIndex = i;
		memcpy(&sndPacket.answer.serviceAddr, &walkData->serviceAddr, 
				sizeof(sndPacket.answer.serviceAddr));
				
		offset = 0;
		strcpy(sndPacket.answer.serviceType + offset, walkData->serviceType);
		offset += strlen(walkData->serviceType) + 1;
		strcpy(sndPacket.answer.serviceType + offset, walkData->serviceName);
		offset += strlen(walkData->serviceName) + 1;
		strcpy(sndPacket.answer.serviceType + offset, walkData->serviceInfo);
		offset += strlen(walkData->serviceInfo) + 1;
		qTraceIfError(socket->Write((Byte8 *) &sndPacket,
				((Byte8 *) sndPacket.answer.serviceType - 
				(Byte8 *) &sndPacket.answer) + offset, remoteAddr), 0);
		found = true;
        }
        
    if (!found) qThrowIfError(SendAckPacket(socket, remoteAddr,
    		transactionID), 0);
   	

/* CATCH */
	qCatch();
	return(0);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 SendAckPacket(DatagramSocket *socket, SocketAddressPtr remoteAddr,
		UInt32 transactionID) {
	SInt32 error = eSocketNoError;
	Packet sndPacket;
	
	sndPacket.ack.transactionType = eTransactionAck;
	sndPacket.ack.transactionID = transactionID;
	sndPacket.ack.reserved[0] = 0;
	sndPacket.ack.reserved[1] = 0;
	qThrowIfError(socket->Write((Byte8 *) &sndPacket, sizeof(sndPacket.ack), 
			remoteAddr), 0);

/* CATCH */
	qCatch();

	return(error);
	}

